@@ -3,3 +3,5 @@ COMMAND_REFACTOR_EXTRACT_VARIABLE = "pylsp_rope.refactor.extract.variable"
 COMMAND_REFACTOR_INLINE = "pylsp_rope.refactor.inline"
 COMMAND_REFACTOR_USE_FUNCTION = "pylsp_rope.refactor.use_function"
 COMMAND_REFACTOR_METHOD_TO_METHOD_OBJECT = "pylsp_rope.refactor.method_to_method_object"
+COMMAND_REFACTOR_LOCAL_TO_FIELD = "pylsp_rope.refactor.local_to_field"
+COMMAND_SOURCE_ORGANIZE_IMPORT = "pylsp_rope.source.organize_import"
