@@ -4,7 +4,14 @@ from typing import List
 
 from pylsp import hookimpl
 from pylsp.lsp import MessageType
-from rope.refactor import extract, inline, method_object, usefunction
+from rope.refactor import (
+    extract,
+    inline,
+    method_object,
+    usefunction,
+    localtofield,
+    importutils,
+)
 
 from pylsp_rope import typing, commands
 from pylsp_rope.project import (
@@ -68,6 +75,10 @@ def pylsp_code_actions(
         end_offset = current_document.offset_at_position(range["end"])
         selected_text = document.source[start_offset:end_offset]
 
+    project = get_project(workspace)
+    for resource in get_resources(workspace, workspace.documents.keys()):
+        project.pycore._invalidate_resource_cache(resource)
+
     commands = {
         "Extract method": CommandRefactorExtractMethod(
             workspace,
@@ -100,6 +111,15 @@ def pylsp_code_actions(
             document_uri=document.uri,
             position=info.position,
         ),
+        "Convert local variable to field": CommandRefactorLocalToField(
+            workspace,
+            document_uri=document.uri,
+            position=info.position,
+        ),
+        "Organize import": CommandSourceOrganizeImport(
+            workspace,
+            document_uri=document.uri,
+        ),
     }
 
     return [
@@ -140,6 +160,10 @@ class Command:
         self.__dict__.update(**arguments)
 
     def __call__(self):
+        rope_changeset = self.get_changes()
+        apply_rope_changeset(self.workspace, rope_changeset)
+
+    def get_changes(self):
         pass
 
     def validate(self, info):
@@ -183,7 +207,7 @@ class CommandRefactorExtractMethod(Command):
     # def _is_valid(self, info):
     #     ...
 
-    def __call__(self):
+    def get_changes(self):
         current_document, resource = get_resource(self.workspace, self.document_uri)
 
         refactoring = extract.ExtractMethod(
@@ -195,7 +219,7 @@ class CommandRefactorExtractMethod(Command):
         rope_changeset = refactoring.get_changes(
             extracted_name="extracted_method",
         )
-        apply_rope_changeset(self.workspace, rope_changeset)
+        return rope_changeset
 
 
 class CommandRefactorExtractVariable(Command):
@@ -209,7 +233,7 @@ class CommandRefactorExtractVariable(Command):
         # FIXME: requires rope.refactor.extract._ExceptionalConditionChecker for proper checking
         ast.parse(info.selected_text, mode="eval")
 
-    def __call__(self):
+    def get_changes(self):
         current_document, resource = get_resource(self.workspace, self.document_uri)
 
         refactoring = extract.ExtractVariable(
@@ -221,7 +245,7 @@ class CommandRefactorExtractVariable(Command):
         rope_changeset = refactoring.get_changes(
             extracted_name="extracted_variable",
         )
-        apply_rope_changeset(self.workspace, rope_changeset)
+        return rope_changeset
 
 
 class CommandRefactorInline(Command):
@@ -238,7 +262,7 @@ class CommandRefactorInline(Command):
             offset=info.current_document.offset_at_position(info.position),
         )
 
-    def __call__(self):
+    def get_changes(self):
         current_document, resource = get_resource(self.workspace, self.document_uri)
 
         refactoring = inline.create_inline(
@@ -247,7 +271,7 @@ class CommandRefactorInline(Command):
             offset=current_document.offset_at_position(self.position),
         )
         rope_changeset = refactoring.get_changes()
-        apply_rope_changeset(self.workspace, rope_changeset)
+        return rope_changeset
 
 
 class CommandRefactorUseFunction(Command):
@@ -264,7 +288,7 @@ class CommandRefactorUseFunction(Command):
             offset=info.current_document.offset_at_position(info.position),
         )
 
-    def __call__(self):
+    def get_changes(self):
         current_document, resource = get_resource(self.workspace, self.document_uri)
 
         refactoring = usefunction.UseFunction(
@@ -275,7 +299,7 @@ class CommandRefactorUseFunction(Command):
         rope_changeset = refactoring.get_changes(
             resources=get_resources(self.workspace, getattr(self, "documents", None)),
         )
-        apply_rope_changeset(self.workspace, rope_changeset)
+        return rope_changeset
 
 
 class CommandRefactorMethodToMethodObject(Command):
@@ -292,7 +316,7 @@ class CommandRefactorMethodToMethodObject(Command):
             offset=info.current_document.offset_at_position(self.position),
         )
 
-    def __call__(self):
+    def get_changes(self):
         current_document, resource = get_resource(self.workspace, self.document_uri)
 
         refactoring = method_object.MethodObject(
@@ -301,4 +325,48 @@ class CommandRefactorMethodToMethodObject(Command):
             offset=current_document.offset_at_position(self.position),
         )
         rope_changeset = refactoring.get_changes(classname="NewMethodObject")
-        apply_rope_changeset(self.workspace, rope_changeset)
+        return rope_changeset
+
+
+class CommandRefactorLocalToField(Command):
+    name = commands.COMMAND_REFACTOR_LOCAL_TO_FIELD
+    kind: CodeActionKind = "refactor.rewrite"
+
+    document_uri: DocumentUri
+    position: typing.Range
+
+    def validate(self, info):
+        localtofield.LocalToField(
+            project=self.project,
+            resource=info.resource,
+            offset=info.current_document.offset_at_position(self.position),
+        )
+
+    def get_changes(self):
+        current_document, resource = get_resource(self.workspace, self.document_uri)
+
+        refactoring = localtofield.LocalToField(
+            project=self.project,
+            resource=resource,
+            offset=current_document.offset_at_position(self.position),
+        )
+        rope_changeset = refactoring.get_changes()
+        return rope_changeset
+
+
+class CommandSourceOrganizeImport(Command):
+    name = commands.COMMAND_SOURCE_ORGANIZE_IMPORT
+    kind: CodeActionKind = "source.organizeImports"
+
+    document_uri: DocumentUri
+
+    def get_changes(self):
+        current_document, resource = get_resource(self.workspace, self.document_uri)
+
+        organizer = importutils.ImportOrganizer(
+            project=self.project,
+        )
+        rope_changeset = organizer.organize_imports(
+            resource=resource,
+        )
+        return rope_changeset
